/**
 * JASMINe
 * Copyright (C) 2007-2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MuleWrapper.java 4798 2009-08-07 12:30:47Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eventswitch.osgi;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.resource.spi.ResourceAdapterInternalException;

import org.mule.MuleManager;
import org.mule.config.ConfigurationBuilder;
import org.mule.config.ReaderResource;
import org.mule.config.builders.MuleXmlConfigurationBuilder;
import org.mule.umo.manager.UMOManager;
import org.ow2.jasmine.monitoring.eventswitch.manager.EventSwitchManager;
import org.ow2.jonas.jmx.JmxService;
import org.ow2.jonas.lib.bootstrap.JProp;

/**
 * Wrapper class for Mule.
 */
public class MuleWrapper {

    /**
     * The JOnAS JMX service.
     */
    private JmxService jmxService;

    /**
     * EventSwitchManager. The instance is unique.
     */
    private EventSwitchManager manager = null;

    /**
     * JMX Server.
     */
    private MBeanServer mbeanServer = null;

    /**
     * OBjectname of the Manager.
     */
    private ObjectName objName = null;

    /**
     * Configures Mule, which will automatically launch it.
     *
     * @throws Exception When something wrong happens ...
     */
    public void start() throws Exception {

        InputStream inputStream;
        this.manager = EventSwitchManager.getInstance();

        try {
            inputStream = new FileInputStream(new File(JProp.getConfDir(), "eventswitch-config.xml"));
        } catch (FileNotFoundException e) {
            throw new Exception("Cannot read the config file 'eventswitch-config.xml'", e);
        }
        if(inputStream == null) {
            throw new Exception("Cannot read the config file 'eventswitch-config.xml'");
        }
        ClassLoader lastClassLoader =
            Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(getClass().getClassLoader());
        try {
            if (MuleManager.isInstanciated()) {
                throw new Exception("A Mule instance is already present");
            }

            try {
                ConfigurationBuilder builder = new MuleXmlConfigurationBuilder();
                ReaderResource configReader = new ReaderResource("eventswitch", new InputStreamReader(inputStream));
                UMOManager umo = builder.configure(new ReaderResource[]{configReader}, null);

                this.manager.setMuleManager(umo);

                try {


                    this.mbeanServer = jmxService.getJmxServer();
                    String domainName = jmxService.getDomainName();
                    String serverName = jmxService.getJonasServerName();

                    this.objName = new ObjectName(domainName + ":type=EventSwitch,name=manager,J2EEServer=" +serverName );

                    this.mbeanServer.registerMBean(this.manager, this.objName);

                } catch (Exception e) {
                    Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Failed registering EventSwitchManager MBean");
                    throw new ResourceAdapterInternalException("Failed registering EventSwitchManager MBean", e);
                }

            } catch(Exception e) {
                MuleManager.getInstance().dispose();
                throw new Exception("Failed configuring Mule", e);
            }
        } finally {
            Thread.currentThread().setContextClassLoader(lastClassLoader);
        }
    }

    /**
     * Disposes of Mule.
     */
    public void stop() {

        try {
            this.mbeanServer.unregisterMBean(this.objName);
        } catch (Exception e) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Failed unregistering EventSwitchManager MBean");
        }

        MuleManager.getInstance().dispose();
        EventSwitchManager.deleteInstance();
    }

    public void setJmxService(final JmxService jmxService) {
        this.jmxService = jmxService;
    }

}
